/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CompileRulePackageCommand.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.utils.getQualifiedClassName;

import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;

import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.business.NotificationEditorDelegate;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.NotificationEditorModelLocator;
import org.ow2.kerneos.common.event.KerneosNotificationEvent;

/**
 * @author durieuxp
 */
[Event(name="serverSideException", type="org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent")]
public class AbstractCommand implements ICommand, IResponder {

    protected const model:NotificationEditorModelLocator = NotificationEditor.getInstance().getModel();
    protected const dispatcher:CairngormEventDispatcher = NotificationEditor.getInstance().getDispatcher();
    protected const delegate:NotificationEditorDelegate = model.getDelegate();

    protected var errormessage:String = "Could not call the server";

    public function AbstractCommand() {
    }

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    public function execute(event:CairngormEvent):void {
        // To be overridden
    }

    /**
     * Get the result of the java side. this method is called on each event
     * returned from the Java side.
     */
    public function result(event:Object):void {
        // To be overridden
    }

    /**
     * Handle fault messages
     */
    public function fault(event:Object):void {
        // Retrieve the fault event
        var faultEvent:FaultEvent = FaultEvent(event);

        // Tell the view and let it handle this
        var serverSideExceptionEvent:ServerSideExceptionEvent =
                new ServerSideExceptionEvent(
                        ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION,
                        new ServerSideException(errormessage,
                                "The operation could not be performed."
                                        + "\n" + faultEvent.fault.faultCode
                                        + "\n" + faultEvent.fault.faultString,
                                faultEvent.fault.getStackTrace()));
        dispatcher.dispatchEvent(serverSideExceptionEvent);
    }

    protected function debug(mess:String):void {
        dispatcher.dispatchEvent(
                new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION,
                        (getQualifiedClassName(this).split("::"))[1] + ": " + mess, KerneosNotificationEvent.DEBUG));
    }
}
}
