/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ImportRulePackageCommand.as 8512 2011-07-12 07:00:31Z diorcety $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.command {
import com.adobe.cairngorm.control.CairngormEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.ServerFilePathEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.SynchronizedRulePackageEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.AbstractRulePackageVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.IRulePackageVO;

/**
 * Import the file as a rule package
 *
 * @author Julien Nicoulaud
 */
public class ImportRulePackageCommand extends AbstractCommand {

    public function ImportRulePackageCommand() {
    }

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    override public function execute(event:CairngormEvent):void {
        // Retrieve the target object
        var filePath:String = (event as ServerFilePathEvent).filePath;

        // Dispatch a notification to Kerneos
        debug("Importing a rule package");

        delegate.responder = this;
        delegate.importRulePackage(filePath, model.ruleEngineName);
    }

    /**
     * Get the result of the java side. this method is called on each event
     * returned from the Java side.
     */
    override public function result(event:Object):void {
        // Retrieve the rule package
        var rulePackage:IRulePackageVO = (event as ResultEvent).result as IRulePackageVO;

        // Complete the rule package properties
        rulePackage.ruleEngineInstanceName = model.ruleEngineName;
        rulePackage.state = AbstractRulePackageVO.DEACTIVATED;
        rulePackage.previousSystemName = null;

        // Wrap the rule package in a SynchronizedRulePackage
        var newSynchronizedRulePackage:SynchronizedRulePackage =
                new SynchronizedRulePackage(rulePackage, SynchronizedRulePackageState.MODIFIED);

        // Add it if not already done
        model.ruleEditorModel.addRulePackage(newSynchronizedRulePackage);

        // Launch a test compilation
        dispatcher.dispatchEvent(
                new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE,
                        newSynchronizedRulePackage)
        );
    }

}
}
