/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LogicalExpressionEditorPM.as 4869 2009-08-11 11:14:12Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.model.presentation.editor.visual
{
import flash.events.Event;
import flash.events.EventDispatcher;
import flash.utils.getDefinitionByName;

import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.notificationeditor.event.ElementChoiceEvent;

import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.*;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.AddElementChoiceWizard;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.ILHSElementEditor;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.LHSCodeSnippetEditor;
import org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.LogicalExpressionEditor;
    
/**
* Presentation model for the visual rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class LogicalExpressionEditorPM extends EventDispatcher
{
    
    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The edited logical expression
    */
    public var editedLogicalExpression : ILogicalExpressionVO;
    
    /**
    * The Operands elements editors
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual.ILHSElementEditor')]
    public var operandsEditors : ArrayCollection = new ArrayCollection();
    
    
    // =========================================================================
    // Constructors
    // =========================================================================
    
    /**
    * Build the Presentation Model
    */
    public function LogicalExpressionEditorPM(editedLogicalExpression:ILogicalExpressionVO)
    {
        // Initialize variables
        this.editedLogicalExpression = editedLogicalExpression;
        
        // Initialize the editors collection
        initializeOperandsEditors();
        
        // Watch the lists changes
        editedLogicalExpression.operands.addEventListener(CollectionEvent.COLLECTION_CHANGE,
                                                          operandsCollectionChangeHandler);
    }
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Create a new Operand
    */
    public function addOperand():void
    {
        var choiceWizard : AddElementChoiceWizard = new AddElementChoiceWizard('ILHSElementVO');
        choiceWizard.addEventListener(ElementChoiceEvent.CHOOSE_ELEMENT_TYPE,addLHSElementChosen);
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Initialize the collection of Operands editors
    */
    private function initializeOperandsEditors():void
    {
        operandsEditors.removeAll();
        for each (var operand : ILHSElementVO in editedLogicalExpression.operands) {
            operandsEditors.addItem(createNewOperandEditor(operand));
        }
    }
    
    /**
    * Create a new editor for an operand
    */
    private function createNewOperandEditor(operand:ILHSElementVO):ILHSElementEditor
    {
        // Declare a new editor
        var newEditor : ILHSElementEditor;
        
        // Instanciate an editor depending on the type of the object
        if (operand is LHSCodeSnippetVO) {
            newEditor = new LHSCodeSnippetEditor();
        } else if (operand is LogicalExpressionVO) {
            newEditor = new LogicalExpressionEditor();
        } else {
            throw new Error('No editor available for this kind of object');
        }
        newEditor.editedLHSElement = operand;
        
        // Listen to demands from the editor
        newEditor.addEventListener("deleteMe", deleteMeHandler);

        return newEditor;
    }
    
    
    // =========================================================================
    // Event handlers
    // =========================================================================
    
    /**
    * When the Operands elements collection changes
    */
    private function operandsCollectionChangeHandler(event:CollectionEvent):void
    {
        switch(event.kind) {
            case CollectionEventKind.ADD:
               for each (var newOperand : ILHSElementVO in event.items) {
                   operandsEditors.addItem(createNewOperandEditor(newOperand));
               }
               break;
            case CollectionEventKind.REMOVE:
                for each (var removedOperand : ILHSElementVO in event.items) {
                    for each (var editor : ILHSElementEditor in operandsEditors) {
                        if (editor.editedLHSElement == removedOperand) {
                            operandsEditors.removeItemAt(operandsEditors.getItemIndex(editor));
                        }
                    }
                }
                break;
            default:
                break;
        }
    }
    
    /**
    * When an element type has been chosen to be added to the LHS
    */
    private function addLHSElementChosen(event:ElementChoiceEvent):void
    {
        var newOperandClass:Class = getDefinitionByName(event.elementType) as Class;
        editedLogicalExpression.operands.addItem(new newOperandClass());
    }
    
    /**
    * When a child asks for deletion
    */
    private function deleteMeHandler(event:Event):void
    {
        var targetEditor : Object = event.target;
        editedLogicalExpression.operands.removeItemAt(editedLogicalExpression.operands.getItemIndex(
            (targetEditor as ILHSElementEditor).editedLHSElement)
        );
    }
}
}
