package org.ow2.jasmine.monitoring.eos.notificationeditor.utils {
import mx.formatters.Formatter;

/**
 * A util class to format a rule package name from a readable name
 * to a package system name.
 *
 * @author Julien Nicoulaud
 * @example
 *   "My Pack@ge_made on 23/06" => "org.ow2.jasmine.rules.packages.MyPackgemadeon2306"
 */
public class RulePackageNameFormatter extends Formatter {
    // =========================================================================
    // Variables
    // =========================================================================

    /**
     * The provided operations
     */
    public static var FORMAT_SYSTEM_NAME:String = "formatSystemName";

    /**
     * The default prefix for rule package names
     */
    public static var SYSTEM_NAME_PREFIX:String = "org.ow2.jasmine.rules.packages.";

    /**
     * The default operation procided by the formatter
     */
    public var operation:String = FORMAT_SYSTEM_NAME;


    // =========================================================================
    // Constructor
    // =========================================================================

    /**
     * Build a new rule package name formatter
     */
    public function RulePackageNameFormatter() {
        // Call super class method
        super();
    }


    // =========================================================================
    // Public methods
    // =========================================================================

    /**
     * Format a given String
     */
    override public function format(value:Object):String {
        // Validate value
        if (value.length == 0) {
            error = "Name is empty";
            return "";
        }

        // Format the string
        switch (operation) {
            case FORMAT_SYSTEM_NAME:
                var forbiddenCharacters:RegExp = /[^a-z0-9]/gi;
                var formattedName:String = (value as String).replace(forbiddenCharacters, "");
                if (formattedName == "") {
                    return "";
                } else {
                    return SYSTEM_NAME_PREFIX + formattedName;
                }
                break;
            default :
                error = "Invalid operation name :\"" + operation + "\"";
                return ""
        }
    }
}
}
