/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AddElementChoiceWizard.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.view.editor.visual
{

import flash.display.DisplayObject;
import flash.events.EventDispatcher;
import flash.utils.getDefinitionByName;

import mx.collections.ArrayCollection;
import mx.core.Application;
import mx.core.FlexGlobals;
import mx.managers.PopUpManager;

import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IRuleElementVO;
import org.ow2.jasmine.monitoring.eos.notificationeditor.event.ElementChoiceEvent;
import org.ow2.jasmine.monitoring.eos.notificationeditor.model.domain.RuleEditorModel;

/**
* A wizard to make the user choose an element type to add to the rule
* 
* @author Julien Nicoulaud
* @internal Potentially chosen classes are all instantiated, this could be done better...
*/
[Event(name='chooseElementType',type='org.ow2.jasmine.monitoring.eos.notificationeditor.event.ElementChoiceEvent')]
public class AddElementChoiceWizard extends EventDispatcher
{
    // =========================================================================
    // Properties
    // =========================================================================
    
    /**
    * The classes that can be chosen be the user
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IRuleElementVO')]
    public var validChoices : ArrayCollection = new ArrayCollection();
    
    
    // =========================================================================
    // Constructor
    // =========================================================================
        
    /**
    * Build a new AddElementChoiceWizard
    */
    public function AddElementChoiceWizard(elementType:String)
    {
        // Resolve the valid choices to display
        resolveValidChoices(elementType);
        
        // Display a window where the user will make his choice
        showChoiceWindow();
    }
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Used by the window to return the result. Must be a full class name such as
    * "rulepackage.visual.RHSCodeSnippet"
    */
    public function set result(value:String):void
    {
        dispatchEvent(new ElementChoiceEvent(ElementChoiceEvent.CHOOSE_ELEMENT_TYPE,value));
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Resolve the available choices and fill the array of choices
    */
    private function resolveValidChoices(elementType:String):void
    {
        var model : RuleEditorModel = NotificationEditor.getInstance().getModel().ruleEditorModel;
        var elementTypeClass : Class = getDefinitionByName(RuleEditorModel.visualElementTypesPackage + elementType) as Class;
        
        // Select in the allowed element types list the ones that implement/extend
        // the given type
        for each (var potentialValidChoice : String in model.allowedVisualElementTypes) {
            var potentialValidChoiceClass : Class = getDefinitionByName(RuleEditorModel.visualElementTypesPackage + potentialValidChoice) as Class;
            var potentialValidChoiceInstance : IRuleElementVO = new(potentialValidChoiceClass)() as IRuleElementVO;
            if (potentialValidChoiceInstance is elementTypeClass) {
                validChoices.addItem(potentialValidChoiceInstance);
            }
        }
    }
    
    /**
    * Display a window to make the user choose in the available chocies array
    */
    private function showChoiceWindow():void
    {
        if (addElementChoiceWindow == null) {
            
            // Create a window to force the user choosing a strategy
            var addElementChoiceWindow:AddElementChoiceWindow = new AddElementChoiceWindow();
            addElementChoiceWindow.choices = validChoices;
            addElementChoiceWindow.choiceWizard = this;
            
            PopUpManager.addPopUp(addElementChoiceWindow,
                                  FlexGlobals.topLevelApplication as DisplayObject,
                                  true);
            PopUpManager.centerPopUp(addElementChoiceWindow);
            
        }
    }
}
}
