/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: IRulePackageVO.as 8807 2011-09-06 14:34:57Z diorcety $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage
{

import mx.collections.ArrayCollection;

/**
* A rule package
*
* @author Julien Nicoulaud
*
*/
[Bindable]
public interface IRulePackageVO extends IRulePackageMember
{
	
    // =========================================================================
    // Properties
    // =========================================================================
	
    // Data values
    
    /**
    * The unique system name of the rule package. Identifies the object, must be
    * different for each rule package. Automatically generated from the rule
    * package "display" name.
    * @example "org.ow2.jasmine.monitoring.rules.example.ExamplePackage"
    */
    function get systemName():String;
	function set systemName(value:String):void;

    /**
     * The previous systemName of the package in the case where it 
     * was modified. Used to identify the old rule package.
     */
    function get previousSystemName():String;
    function set previousSystemName(value:String):void;
    
    /**
    * The displayed name of the rule package. This is the name that is choosed by
    * the user. Used to generate automatically the systemName property.
    * @example "Example Package"
    */
    function get displayName():String;
    function set displayName(value:String):void;
    
    /**
     * The name of the rule engine instance on which the rule is executed.
     */
    function get ruleEngineInstanceName():String;
    function set ruleEngineInstanceName(value:String):void;
    	
    /**
    * The state of the rule package. Values can be:
    *   - Activated
    *   - Deactivated
    *   - Activation requested
    *   - Deactivation requested
    */
    function get state():String;
    function set state(value:String):void;
    
    /**
    * The description of the rule package
    */
    function get description():String;
    function set description(value:String):void;
    
    
    // Transient properties
    
    /**
    * The validity of the name
    */
    [Transient]
    function get nameIsValid():Boolean;
    function set nameIsValid(value:Boolean):void
    
    /**
    * The availability of the name
    */
    [Transient]
    function get nameIsAvailable():Boolean;
    function set nameIsAvailable(value:Boolean):void;
    
    /**
    * The compilation errors
    */
    [Transient]
    function get compilationErrors():ArrayCollection;
    function set compilationErrors(value:ArrayCollection):void;
    
    /**
    * Wether the compilation has been checked since the last change
    */
    [Transient]
    function get compilationChecked():Boolean;
    function set compilationChecked(value:Boolean):void;
    

    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Check if the rule package declares itself as ready to be saved
    * @return null if no issue
    *         a String describing the problem if there is one
    */
    function canBeSaved():String;
    
    /**
    * Get the state to assign for the given target state
    * @example : if the current state is DEACTIVATED,
    *            getStateforTarget(ACTIVATED) == ACTIVATION_REQUESTED
    */    
    function getStateForTarget(value:String):String;
    
}
}
