/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleVO.as 9121 2011-10-10 12:38:53Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual
{

import com.adobe.cairngorm.vo.IValueObject;

import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.AbstractRulePackageMemberVO;
import org.ow2.kerneos.common.util.StringUtils;

/**
* A single rule
*
* @author Julien Nicoulaud
*/
public class RuleVO extends AbstractRulePackageMemberVO
                    implements IRuleVO, IValueObject
{
	
	// =========================================================================
	// Properties
	// =========================================================================
    
    /**
    * The rule name
    */
    private var _name : String = "";
        
    /**
    * The LHS (conditions)
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.ILHSElementVO')]
    private var _lhs : ArrayCollection;
    
    /**
    * The RHS (actions)
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.notificationeditor.vo.rulepackage.visual.IRHSElementVO')]
    private var _rhs : ArrayCollection;

    /**
    * The imports
    */
    [ArrayElementType('String')]
    private var _imports : ArrayCollection;

    /**
    * The variables defined in the rule
    */
    [ArrayElementType('String')]
    private var _variables : ArrayCollection;


    public function RuleVO() {
    }

// =========================================================================
    // Getters and setters
    // =========================================================================
    
    /**
    * Set the rule name
    */
    public function set name(value:String):void
    {
        // Store the old value
        var oldValue : Object = _name;
        
        // Assign the value
        _name = value;

        // Dispatch a property change event
        notifyPropertyChange("name",oldValue,value);
    }
    
    /**
    * Get the rule name
    */
    [Bindable]
    public function get name():String
    {
        return _name;
    }
    
    /**
    * Set the LHS
    */
    public function set lhs(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _lhs;

        // Assign the value
        _lhs = value;
        
        // Bind some properties for each collection item
        for each (var lhsElement : ILHSElementVO in lhs) {
            lhsElement.imports = imports;
            lhsElement.variables = variables;
        }
        
        // Setup the event listener
        _lhs.addEventListener(CollectionEvent.COLLECTION_CHANGE,lhsCollectionChangeHandler);
        
        // Dispatch a property change event
        notifyPropertyChange("lhs",oldValue,value);
    }
    
    /**
    * Get the LHS
    */
    [Bindable]
    public function get lhs():ArrayCollection
    {
        // If the field is not initialized, do it
        if (_lhs == null) {
            _lhs = new ArrayCollection();
            _lhs.addEventListener(CollectionEvent.COLLECTION_CHANGE,lhsCollectionChangeHandler,false,0,true);
        }
        return _lhs;
    }
    
    /**
    * Set the RHS
    */
    public function set rhs(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _rhs;

        // Assign the value
        _rhs = value;

        // Bind some properties for each collection item
        for each (var rhsElement : IRHSElementVO in rhs) {
            rhsElement.imports = imports;
            rhsElement.variables = variables;
        }
        
        // Setup the event listener
        _rhs.addEventListener(CollectionEvent.COLLECTION_CHANGE,rhsCollectionChangeHandler);
        
        // Dispatch a property change event
        notifyPropertyChange("rhs",oldValue,value);
    }
    
    /**
    * Get the RHS
    */
    [Bindable]
    public function get rhs():ArrayCollection
    {
        // If the field is not initialized, do it
        if (_rhs == null) {
            _rhs = new ArrayCollection();
            _rhs.addEventListener(CollectionEvent.COLLECTION_CHANGE,rhsCollectionChangeHandler,false,0,true);
        }
        return _rhs;
    }
    
    /**
    * Set the imports collection
    */    
    public function set imports(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _imports;
        
        // Assign the value
        _imports = value;
        
        // Propagate the change to the children
        for each (var lhsElement : ILHSElementVO in lhs) {
            lhsElement.imports = imports;
        }
        for each (var rhsElement : IRHSElementVO in rhs) {
            rhsElement.imports = imports;
        }
        
        // Dispatch a property change event
        notifyPropertyChange("imports",oldValue,value);
    }
    
    /**
    * Get the imports collection
    */
    [Transient]
    [Bindable]
    public function get imports():ArrayCollection
    {
        // Initialize the property if needed
        if (_imports == null) {
            _imports = new ArrayCollection();
        }
        return _imports;
    }
    
    /**
    * Set the variables collection
    */    
    public function set variables(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _variables;
        
        // Assign the value
        _variables = value;

        // Propagate the change to the children
        for each (var lhsElement : ILHSElementVO in lhs) {
            lhsElement.variables = variables;
        }
        for each (var rhsElement : IRHSElementVO in rhs) {
            rhsElement.variables = variables;
        }
        
        // Dispatch a property change event
        notifyPropertyChange("variables",oldValue,value);
    }
    
    /**
    * Get the variables collection
    */
    [Bindable]
    public function get variables():ArrayCollection
    {
        // Initialize the property if needed
        if (_variables == null) {
            _variables = new ArrayCollection();
        }
        return _variables;
    }
	
	
    // =========================================================================
    // Implementation of IRulePackageVO
    // =========================================================================
    
    /**
    * Get the generated source in Drools syntax
    */
    override public function getDrlSource():String
    {
    	var source : String = "";
    	
    	// Format the name
    	source += "rule \"" + name + "\"";
    	
        // Format conditions
        source += "\nwhen";
        for each (var lhsElement:ILHSElementVO in lhs) {
            source += "\n" + StringUtils.indent(lhsElement.getDrlSource());
        }
        
        // Format actions
        source += "\nthen";
        for each (var rhsElement:IRHSElementVO in rhs) {
            source += "\n" + StringUtils.indent(rhsElement.getDrlSource());
        }
        
        // Format the suffix
        source += "\nend";
        
        return source;
    }
    
    
    // =========================================================================
    // Events handlers
    // =========================================================================
    
    /**
    * When the LHS elements list changes
    */
    private function lhsCollectionChangeHandler(event:CollectionEvent):void
    {
        // Bind some synchronized variables
        switch (event.kind) {
            case CollectionEventKind.ADD:
                for each (var lhsElement : ILHSElementVO in event.items) {
                    lhsElement.imports = imports;
                    lhsElement.variables = variables;
                }
                break;
        }
        
        // Notify the change
        notifyPropertyChange("lhs",_lhs,lhs);
    }
    
    /**
    * When the RHS elements list changes
    */
    private function rhsCollectionChangeHandler(event:CollectionEvent):void
    {
        // Bind some synchronized variables
        switch (event.kind) {
            case CollectionEventKind.ADD:
                for each (var rhsElement : IRHSElementVO in event.items) {
                    rhsElement.imports = imports;
                    rhsElement.variables = variables;
                }
                break;
        }
        
        // Notify the change
        notifyPropertyChange("rhs",_rhs,rhs);
    }    
}
}
